import sys
import logging
from os.path import join
from PyQt5.QtWidgets import QApplication
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtCore import Qt
from droptopus import config, settings, args
from droptopus.window import MainWindow

class App(QApplication):
    def __init__(self, args=None):
        logging.info("Initializing Droptopus App")
        super(App, self).__init__([config.APP_NAME])
        css = """
    MiniWindow,
    DropTitleBar {
        Background: transparent;
    }
    MainWindow {
        font-size: 12px;
        color: black;
        Background: transparent;
        color: white;
        font:12px bold;
        font-weight: bold;
        border-top-left-radius: 10px;
        border-top-right-radius: 10px;
        height: 30px;
    }
    DropTitleBar > QToolButton {
        Background: transparent;
        font-size: 11px;
    }
    DropTitleBar > QToolButton:focus {
        border: none;
        outline: none;
    }
    QLabel {
        color:white;
        Background: transparent;
    }
    EditItemForm QLabel,
    QMessageBox QLabel,
    QInputDialog QLabel {
        color: #333;
    }
    AboutDialog,
    DropTitleBar,
    DropFrame {
        Background: rgba(0,0,0,70%);
        color:white;
        font:13px ;
        font-weight:bold;
    }
    AboutDialog,
    DropTitleBar,
    DropFrame {
        border-radius: 10px;
    }
    AboutDialog QPushButton {
        background-color: gray;
    }
    AboutDialog QLabel#title {
        font-size: 16px;
        font-weight: bold;
    }
    DropWidget {
        background-color: rgba(0,0,0,0%);
    }
    DropWidget[draggedOver=true],
    DropWidget[hover=true] {
        background-color: rgba(255,255,255,5%);
    }
    """
        self.setStyleSheet(css)
        self.mainWindow = MainWindow()
        self.mainWindow.show()

        iconFilepath = join(config.ASSETS_DIR, "droptopus.png")
        icon = QIcon(
            QPixmap(iconFilepath).scaled(
                50, 50, Qt.KeepAspectRatio, Qt.SmoothTransformation
            )
        )
        self.setWindowIcon(icon)



import argparse
import logging

parser = argparse.ArgumentParser()
parser.add_argument(
    "-d",
    "--debug",
    help="Print lots of debugging statements",
    action="store_const",
    dest="loglevel",
    const=logging.DEBUG,
    default=logging.WARNING,
)
parser.add_argument(
    "-v",
    "--verbose",
    help="Be verbose",
    action="store_const",
    dest="loglevel",
    const=logging.INFO,
)
args = parser.parse_args()

logging.basicConfig(
    format="%(asctime)s | %(message)s", filename="debug.log", level=args.loglevel
)



import os
import logging

from droptopus import config, settings

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QFileDialog,
    QFormLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
)
from PyQt5.QtGui import QPixmap


class EditItemForm(QDialog):
    def __init__(self, item, parent=None):
        super(EditItemForm, self).__init__(parent)
        l1 = QLabel("Name:")
        name = QLineEdit()

        l2 = QLabel("Description:")
        desc = QLineEdit()

        l3 = QLabel("Icon:")
        icon = QLabel()
        btn_icon = QPushButton("...")
        btn_icon.setFixedWidth(50)
        btn_icon.clicked.connect(self.onChangeIcon)

        l4 = QLabel("Target Path:")
        path = QLineEdit()
        path.setReadOnly(True)
        btn_path = QPushButton("...")
        btn_path.setFixedWidth(50)
        btn_path.clicked.connect(self.onChangePath)

        layout = QFormLayout(self)
        layout.addRow(l1, name)
        layout.addRow(l2, desc)
        row = QHBoxLayout()
        row.addWidget(icon)
        row.addWidget(btn_icon)
        layout.addRow(l3, row)
        row = QHBoxLayout()
        row.addWidget(path)
        row.addWidget(btn_path)
        layout.addRow(l4, row)
        buttons = QDialogButtonBox(
            QDialogButtonBox.Ok | QDialogButtonBox.Cancel, Qt.Horizontal, self
        )
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        layout.addRow(buttons)

        self.icon = icon
        self.name = name
        self.path = path
        self.desc = desc

        self.loadItem(item)

    def loadItem(self, item):
        self.icon.setPixmap(
            QPixmap(item["icon"]).scaled(
                50, 50, Qt.KeepAspectRatio, Qt.SmoothTransformation
            )
        )
        self.name.setText(item["name"])
        self.desc.setText(item["desc"])
        self.path.setText(item["path"])
        self.item = item

    def onChangeIcon(self):
        icon_filepath, _filter = QFileDialog.getOpenFileName(
            self, "Choose Icon", os.path.dirname(self.item["icon"])
        )
        if icon_filepath:
            icon_size = 15
            self.icon.setPixmap(
                QPixmap(icon_filepath).scaled(
                    icon_size, icon_size, Qt.KeepAspectRatio, Qt.SmoothTransformation
                )
            )
            self.item["icon"] = icon_filepath

    def onChangePath(self):
        path = self.item["path"] if len(self.item["path"]) else os.path.expanduser("~")
        if self.item["type"] == "dir":
            path = QFileDialog.getExistingDirectory(self, "Choose a directory", path)
            if path:
                self.path.setText(path)
                self.item["path"] = path
        else:
            path, _filter = QFileDialog.getOpenFileName(self, "Open file", path)
            if path:
                self.path.setText(path)
                self.item["path"] = path

    def validate(self):
        return True

    def accept(self):
        if not self.validate():
            QMessageBox.critical(
                self, "Error", "\n".join(self.validation_errors), QMessageBox.Ok
            )
            return

        self.item["name"] = self.name.text()
        self.item["desc"] = self.desc.text()
        settings.writeItem(self.item)
        self.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Droptopus Main Window

author: Djordje Ungar
website: djordjeungar.com
"""
import os
from os.path import isfile, isdir, join, expanduser
import sys

from droptopus import config, settings, utils, __version__
from droptopus.widgets import DropFrame, events

from PyQt5.QtGui import QIcon, QPainter, QPixmap
from PyQt5.QtCore import QEvent, QSettings, QSize, Qt, QPoint, QTimer
from PyQt5.QtWidgets import (
    QApplication,
    QDesktopWidget,
    QDialog,
    QDialogButtonBox,
    QFileDialog,
    QFormLayout,
    QFrame,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMainWindow,
    QMenu,
    QMessageBox,
    QPushButton,
    QSizePolicy,
    QStackedWidget,
    QStyle,
    QStyleOption,
    QToolButton,
    QVBoxLayout,
    QWidget,
)


class MiniWindow(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self.parent = parent
        icon = join(config.ASSETS_DIR, "droptopus.png")
        self.pixmap = QPixmap(icon).scaled(
            100, 100, Qt.KeepAspectRatio, Qt.SmoothTransformation
        )
        self.icon_width = self.pixmap.width()
        self.icon_height = self.pixmap.height()
        self.setFixedWidth(self.icon_width)
        self.setFixedHeight(self.icon_height)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.drawPixmap(event.rect(), self.pixmap)

    def sizeHint(self):
        return QSize(self.icon_width, self.icon_height)


class DarkDialog(QDialog):
    def __init__(self, parent):
        super(DarkDialog, self).__init__(parent)
        self.setWindowFlags(
            Qt.FramelessWindowHint
            | Qt.WindowStaysOnTopHint
            | Qt.Tool
            | Qt.WA_MacAlwaysShowToolWindow
        )
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setMouseTracking(True)
        self.is_move_action = False
        self.offset = None

    def mousePressEvent(self, event):
        if not event.button() == Qt.LeftButton:
            return
        self.is_move_action = True
        self.offset = event.pos()

    def mouseReleaseEvent(self, event):
        self.is_move_action = False

    def mouseMoveEvent(self, event):
        if not self.is_move_action:
            return
        x = event.globalX()
        y = event.globalY()
        x_w = self.offset.x()
        y_w = self.offset.y()
        self.move(x - x_w, y - y_w)

    def paintEvent(self, event):
        opt = QStyleOption()
        opt.initFrom(self)
        painter = QPainter(self)
        self.style().drawPrimitive(QStyle.PE_Widget, opt, painter, self)


class AboutDialog(DarkDialog):
    def __init__(self, parent=None):
        super(AboutDialog, self).__init__(parent)

        main_layout = QVBoxLayout(self)
        margin = 20
        main_layout.setContentsMargins(margin, margin, margin, margin)

        layout = QHBoxLayout()
        # add icon
        icon = join(config.ASSETS_DIR, "droptopus.png")
        label = QLabel()
        label.setPixmap(
            QPixmap(icon).scaled(150, 150, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        )
        layout.addWidget(label)

        info_layout = QVBoxLayout()
        # add title
        label = QLabel()
        label.setObjectName("title")
        label.setText("Droptopus")
        info_layout.addWidget(label)
        label = QLabel()
        label.setText("v" + __version__.__version__)
        info_layout.addWidget(label)
        info_layout.addStretch()
        # about text
        self.label = QLabel(self)
        self.label.setOpenExternalLinks(True)
        self.label.setText(
            """
            <div><strong>Author:</strong> Djordje Ungar</div>
            <div><strong>Source:</strong> <a href="https://github.com/ArtBIT/Droptopus" style="color:white">GitHub</a></div>
            <div><strong>Icons:</strong> <a href="http://icons8.com/license" style="color:white">Icons8</a></div>
            <br>
        """
        )
        info_layout.addWidget(self.label)
        info_layout.addStretch()
        layout.addLayout(info_layout)
        layout.addStretch()

        main_layout.addLayout(layout)

        # OK and Cancel buttons
        buttons = QDialogButtonBox(QDialogButtonBox.Ok, Qt.Horizontal, self)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        main_layout.addWidget(buttons)
        self.resize(300, 200)


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent)
        self.settings = QSettings()

        self.is_visible = True
        self.is_expanded = True
        self.is_move_action = False
        self.should_confirm_close = False
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint | Qt.Tool)
        self.setAttribute(Qt.WA_NoSystemBackground, True)
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setAttribute(Qt.WA_QuitOnClose)
        self.setWindowIcon(QIcon(join(config.ASSETS_DIR, "droptopus.png")))
        self.setWindowTitle("Droptopus")

        self.miniwin = MiniWindow(self)
        self.frame = DropFrame(self)
        self.frame.show()

        self.readSettings()

        self.content = QStackedWidget()
        self.setCentralWidget(self.content)
        self.content.addWidget(self.frame)
        self.content.addWidget(self.miniwin)

        self.setAcceptDrops(True)
        self.setMouseTracking(True)
        self.collapse()

    def contextMenuEvent(self, event):
        menu = QMenu(self)
        label = ("Expand", "Collapse")[self.is_expanded]
        expand_action = menu.addAction(label)
        about_action = menu.addAction("About")
        menu.addSeparator()
        quit_action = menu.addAction("Quit")
        action = menu.exec_(self.mapToGlobal(event.pos()))
        if action == expand_action:
            if self.is_expanded:
                self.collapse()
            else:
                self.expand()
        elif action == about_action:
            self.showAbout()
        elif action == quit_action:
            self.should_confirm_close = True
            self.close()

    def expand(self):
        if self.is_expanded:
            return
        self.is_expanded = True
        self.setAcceptDrops(False)
        self.content.hide()
        expanded = self.frame.sizeHint()
        self.setMinimumSize(expanded)
        self.content.setCurrentWidget(self.frame)
        self.content.show()
        self.resize(expanded)

        # on OSX the window will not automatically stay inside the screen like on Linux
        # we have to do it manually
        screen_rect = QDesktopWidget().screenGeometry()
        window_rect = self.frameGeometry()
        intersection = window_rect & screen_rect
        dx = window_rect.width() - intersection.width()
        dy = window_rect.height() - intersection.height()
        unseen = window_rect & intersection
        if dx != 0 or dy != 0:
            if window_rect.left() > screen_rect.left():
                dx = dx * -1
            if window_rect.bottom() > screen_rect.bottom():
                dy = dy * -1
            self.move(window_rect.left() + dx, window_rect.top() + dy)

    def collapse(self):
        if not self.is_expanded:
            return
        self.is_expanded = False
        self.setAcceptDrops(True)
        self.content.hide()
        mini = self.miniwin.sizeHint()
        self.setMinimumSize(mini)
        self.move(self.anchor)
        self.content.setCurrentWidget(self.miniwin)
        self.content.show()
        self.resize(mini)

    def showAbout(self):
        about = AboutDialog(self)
        about.setModal(True)
        about.show()

    def mouseReleaseEvent(self, event):
        self.is_move_action = False

    def mousePressEvent(self, event):
        if not event.button() == Qt.LeftButton:
            return
        self.is_move_action = True
        self.offset = event.pos()

    def mouseMoveEvent(self, event):
        if not self.is_move_action:
            return
        x = event.globalX()
        y = event.globalY()
        x_w = self.offset.x()
        y_w = self.offset.y()
        self.move(x - x_w, y - y_w)
        if self.content.currentWidget() == self.miniwin:
            self.anchor = self.pos()

    def writeSettings(self):
        self.settings.beginGroup("MainWindow")
        self.settings.setValue("anchor", self.anchor)
        self.settings.endGroup()

    def readSettings(self):
        self.settings.beginGroup("MainWindow")
        saved_anchor = self.settings.value("anchor", None)
        if saved_anchor != None:
            self.anchor = saved_anchor
        else:
            rect = QDesktopWidget().screenGeometry()
            mini = self.miniwin.sizeHint()
            self.anchor = QPoint(
                rect.right() - mini.width(), rect.bottom() - mini.height()
            )
        self.settings.endGroup()

    def userReallyWantsToQuit(self):
        if not self.should_confirm_close:
            return True
        reply = QMessageBox.question(
            self,
            "Close Droptopus",
            "Are you sure you want to close the application?",
            QMessageBox.Yes,
            QMessageBox.No,
        )
        return reply == QMessageBox.Yes

    def closeEvent(self, event):
        if self.userReallyWantsToQuit():
            self.writeSettings()
            event.accept()
        else:
            event.ignore()

    # def dragMoveEvent(self, event):
    #    super(MainWindow, self).dragMoveEvent(event)

    def dragEnterEvent(self, event):
        if not self.is_expanded:
            QTimer.singleShot(200, self.expand)
        else:
            super(MainWindow, self).dragEnterEvent(event)

    def mouseDoubleClickEvent(self, event):
        if self.is_expanded:
            self.collapse()
        else:
            self.expand()

    def event(self, evt):
        et = evt.type()
        if et == events.COLLAPSE_WINDOW:
            evt.accept()
            self.collapse()
            return True
        if evt.type() == events.RELOAD_WIDGETS:
            evt.accept()
            if self.is_expanded:
                self.resize(self.sizeHint())
        if et == events.EXPAND_WINDOW:
            evt.accept()
            self.expand()
            return True
        elif et == events.CLOSE_WINDOW:
            evt.accept()
            self.should_confirm_close = True
            self.close()
            return True
        return super(MainWindow, self).event(evt)



from droptopus import config
from PyQt5.QtCore import QCoreApplication, QSettings

QCoreApplication.setOrganizationName(config.ORG_NAME)
QCoreApplication.setOrganizationDomain(config.ORG_DOMAIN)
QCoreApplication.setApplicationName(config.APP_NAME)

settings = QSettings()


def writeItems(items):
    s = settings
    s.remove("items")
    size = len(items)
    s.beginWriteArray("items", size)
    for i in range(size):
        s.setArrayIndex(i)
        value = items[i]
        s.setValue("type", value["type"])
        s.setValue("name", value["name"])
        s.setValue("path", value["path"])
        s.setValue("icon", value["icon"])
        s.setValue("desc", value["desc"])
    s.endArray()


def readItems():
    s = settings
    size = s.beginReadArray("items")
    items = []
    for i in range(size):
        s.setArrayIndex(i)
        items.append(
            {
                "index": i,
                "type": str(s.value("type")),
                "name": str(s.value("name")),
                "path": str(s.value("path")),
                "icon": str(s.value("icon")),
                "desc": str(s.value("desc")),
            }
        )
    s.endArray()
    return items


def readItem(index):
    items = readItems()
    return items[index]


def writeItem(item):
    try:
        index = int(item["index"])
        items = readItems()
        items[item["index"]] = item
        writeItems(items)
        return True
    except ValueError:
        return pushItem(item)
    return False


def pushItem(item):
    items = readItems()
    items.append(item)
    writeItems(items)
    return len(items) - 1


def removeItem(index):
    items = readItems()
    del items[index]
    writeItems(items)



import re
import unicodedata
from PyQt5.QtWidgets import QApplication


def slugify(value):
    """
    Normalizes string, converts to lowercase, removes non-alpha characters,
    and converts spaces to hyphens.
    """
    value = str(unicodedata.normalize("NFKD", value).encode("ascii", "ignore"))
    value = re.sub(r"[^\w\s-]", "", value).strip().lower()
    value = re.sub(r"[-\s]+", "-", value)
    return value


# Remove all items from the layout
def clearLayout(layout):
    if layout != None:
        while layout.count():
            child = layout.takeAt(0)
            if child.widget() is not None:
                child.widget().deleteLater()
            elif child.layout() is not None:
                clearLayout(child.layout())


def propagateEvent(self, evt):
    # QEvent.accepted is True by default
    evt.setAccepted(False)
    app = QApplication.instance()
    target = self
    while target:
        app.sendEvent(target, evt)
        if not evt.isAccepted():
            if hasattr(target, "parent"):
                target = target.parent()
        else:
            target = None
    return evt.isAccepted()


class dotdict(dict):
    """dot.notation access to dictionary attributes"""

    __getattr__ = dict.get
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__



import os
import sys
import re
import magic
import requests
import math
import tempfile
import subprocess
import logging

from os import rename
from shutil import copyfile
from os.path import isfile, isdir, join, expanduser
from droptopus import config, settings, utils
from droptopus.forms import EditItemForm

from PyQt5.QtCore import QEvent, QSettings, QSize, Qt
from PyQt5.QtWidgets import (
    QApplication,
    QDialog,
    QDialogButtonBox,
    QFileDialog,
    QFormLayout,
    QFrame,
    QGridLayout,
    QHBoxLayout,
    QInputDialog,
    QLabel,
    QLineEdit,
    QMenu,
    QMessageBox,
    QPushButton,
    QSizePolicy,
    QStyle,
    QStyleOption,
    QToolButton,
    QVBoxLayout,
    QWidget,
)
from PyQt5.QtGui import QPixmap, QPainter, QIcon, QPalette

events = utils.dotdict(
    {
        "RELOAD_WIDGETS": QEvent.registerEventType(1337),
        "COLLAPSE_WINDOW": QEvent.registerEventType(1338),
        "EXPAND_WINDOW": QEvent.registerEventType(1339),
        "CLOSE_WINDOW": QEvent.registerEventType(1340),
    }
)

re_url = re.compile(
    r"^(?:(?:http|ftp)s?://)?"  # http:// or https://
    r"(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|"  # domain...
    r"localhost|"  # localhost...
    r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"  # ...or ip
    r"(?::\d+)?"  # optional port
    r"(?:/?|[/?]\S+)$",
    re.IGNORECASE,
)


class IconWidget(QWidget):
    def __init__(self, parent, icon, width=48, height=48):
        logging.info("Creating IconWidget with icon: %s", icon)
        super(IconWidget, self).__init__(parent)
        self.pixmap = QPixmap(icon).scaled(
            width, height, Qt.KeepAspectRatio, Qt.SmoothTransformation
        )
        self.setFixedWidth(width)
        self.setFixedHeight(height)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.drawPixmap(event.rect(), self.pixmap)


class BaseDropWidget(QWidget):
    def __init__(self, parent, conf):
        super(BaseDropWidget, self).__init__(parent)
        self.setAcceptDrops(True)
        self.in_context_menu = False

        width = 100
        height = 100
        self.name = conf["name"]
        self.index = conf["index"]
        self.iconpath = conf["icon"]
        self.icon = IconWidget(self, conf["icon"])
        label = QLabel()
        label.setText(conf["name"])
        label.setAlignment(Qt.AlignCenter)
        label.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
        label.setMaximumWidth(width)
        label.setWordWrap(True)
        self.label = label
        layout = QVBoxLayout()
        layout.addWidget(self.icon)
        layout.setAlignment(self.icon, Qt.AlignCenter)
        layout.addWidget(self.label)
        layout.setAlignment(self.label, Qt.AlignCenter)
        self.setLayout(layout)
        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
        self.setFixedWidth(width)
        # self.setFixedSize(width,height)

    def handle(self, context):
        return context

    def setStyleProperty(self, prop, value):
        self.setProperty(prop, value)
        self.style().unpolish(self)
        self.style().polish(self)

    def handleDragEvent(self, event):
        self.setStyleProperty("draggedOver", True)
        if event.mimeData().hasUrls():
            event.accept()
        elif event.mimeData().hasImage():
            event.accept()
        elif event.mimeData().hasText():
            event.accept()
        else:
            event.ignore()

    def dragMoveEvent(self, event):
        self.handleDragEvent(event)

    def dragEnterEvent(self, event):
        self.handleDragEvent(event)

    def dragLeaveEvent(self, event):
        self.setStyleProperty("draggedOver", False)

    def dropEvent(self, event):
        if event.mimeData().hasText():
            event.accept()
            self.handle(event.mimeData().text())
        elif event.mimeData().hasUrls():
            event.accept()
            for url in event.mimeData().urls():
                url = str(url)
                self.handle(url)

        utils.propagateEvent(self, QEvent(events.COLLAPSE_WINDOW))
        QWidget.dropEvent(self, event)

    # Have to override this so that QWidget subclasses
    # can update the background via qss
    def paintEvent(self, event):
        opt = QStyleOption()
        opt.initFrom(self)
        painter = QPainter(self)
        self.style().drawPrimitive(QStyle.PE_Widget, opt, painter, self)


class DropWidget(BaseDropWidget):
    def __init__(self, parent, conf):
        super(DropWidget, self).__init__(parent, conf)
        self.setAttribute(Qt.WA_OpaquePaintEvent)
        self.type = conf["type"]
        self.filepath = conf["path"]
        self.desc = conf["desc"]
        if self.desc:
            self.setToolTip(self.desc)
        self.actions = [
            ("Process Clipboard", self.onPasteFromClipboard),
            ("Process File...", self.onFileOpen),
            ("--", None),
            ("Edit...", self.onEdit),
            ("Remove", self.onDelete),
        ]

    def handle(self, context):
        context = super(DropWidget, self).handle(context)
        if re_url.match(context):
            return self.handle_url(context)
        elif isfile(context):
            return self.handle_filepath(context)
        else:
            return self.handle_text(context)

    def mouseDoubleClickEvent(self, event):
        if sys.platform.startswith("darwin"):
            subprocess.call(("open", self.filepath))
        elif os.name == "nt":
            os.startfile(self.filepath)
        elif os.name == "posix":
            subprocess.call(("xdg-open", self.filepath))

    def enterEvent(self, event):
        self.setStyleProperty("hover", True)

    def leaveEvent(self, event):
        if self.in_context_menu:
            return
        self.setStyleProperty("hover", False)

    def contextMenuEvent(self, event):
        if not self.actions:
            return
        # context menu happens before leaveEvent
        # so we have to have a flag to prevent it
        self.in_context_menu = True
        self.setStyleProperty("hover", True)
        menu = QMenu(self)
        actions = {}
        for k, v in self.actions:
            if k == "--":
                menu.addSeparator()
                continue
            action = menu.addAction(k)
            actions[action] = v

        action = menu.exec_(self.mapToGlobal(event.pos()))
        if action in actions:
            actions[action]()
        self.setStyleProperty("hover", False)
        self.in_context_menu = False

    def onEdit(self):
        """
        Opens and executes the edit form, which allows the user to edit the selected DropTarget
        """
        item = {
            "index": self.index,
            "type": self.type,
            "name": self.name,
            "path": self.filepath,
            "icon": self.iconpath,
            "desc": self.desc,
        }
        form = EditItemForm(item, self)
        form.setModal(True)
        form.exec_()
        utils.propagateEvent(self, QEvent(events.RELOAD_WIDGETS))

    def onDelete(self):
        """
        After confirming the delete action, removes the item from the settings file.
        """
        reply = QMessageBox.question(
            self,
            "Message",
            "Are you sure you want to delete this action?",
            QMessageBox.Yes,
            QMessageBox.No,
        )
        if reply == QMessageBox.Yes:
            settings.removeItem(self.index)
            utils.propagateEvent(self, QEvent(events.RELOAD_WIDGETS))

    def onFileOpen(self):
        """
        Opens a standard file dialog and processess the selected filepath.
        """
        myhome = expanduser("~")
        fname, _filter = QFileDialog.getOpenFileName(self, "Open file", myhome)
        self.handle(fname)

    def onPasteFromClipboard(self):
        """
        Handles clipboard paste event.
        """
        clipboard = QApplication.instance().clipboard()
        self.handle(clipboard.text())


class DirTarget(DropWidget):
    """
    A DropWidget that processes the dropped context and saves it to a directory.
    """

    def handle_filepath(self, filepath):
        """
        Processes passed filepath.
        """
        dest = join(self.filepath, os.path.basename(filepath))
        copyfile(filepath, dest)
        return 0, dest

    def handle_text(self, text):
        """
        Processes passed raw text.
        """
        tmp = next(tempfile._get_candidate_names())
        tmp = join(self.filepath, tmp) + ".txt"
        text_file = open(tmp, "w")
        text_file.write(text.encode("utf8"))
        text_file.close()
        return 0, tmp

    def handle_url(self, url):
        """
        Processes passed URL.
        """
        tmp = tempfile.NamedTemporaryFile(delete=False)
        r = requests.get(url)
        with open(tmp.name, "wb") as fd:
            for chunk in r.iter_content(chunk_size=128):
                fd.write(chunk)
            fd.close()
        ext = magic.from_file(tmp.name, mime=True).split("/")[1]
        dest = join(self.filepath, utils.slugify(url.split("/").pop())[:20] + "." + ext)
        copyfile(tmp.name, dest)
        return 0, dest


class FileTarget(DropWidget):
    """
    A DropWidget that processes the dropped context processes it through a script.
    """

    def handle_filepath(self, filepath):
        """
        Processes passed filepath.
        """
        return subprocessCall([self.filepath, filepath])

    def handle_text(self, text):
        """
        Processes passed raw text.
        """
        return subprocessCall([self.filepath, text])

    def handle_url(self, url):
        """
        Processes passed URL.
        """
        return subprocessCall([self.filepath, url])

    def mouseDoubleClickEvent(self, event):
        subprocessCall([self.filepath])


class CreateTarget(DropWidget):
    """
    A DropWidget that creates a DirTarget or FileTarget
    """

    def __init__(self, parent, conf):
        conf["type"] = "builtin"
        super(CreateTarget, self).__init__(parent, conf)
        self.actions = [
            ("Create Folder Action...", self.onCreateFolderAction),
            ("Create Executable Action...", self.onCreateFileAction),
            # ('Create From Template...', self.onCreateFromTemplate),
        ]

    def onCreateFileAction(self):
        """
        Create a new FileTarget.
        """
        context, _filter = QFileDialog.getOpenFileName(
            self, "Open file", expanduser("~")
        )
        context = str(context)
        if not context:
            return 1, "Missing argument"
        logging.info("Create file target: %s", context)
        name = os.path.basename(context)
        name, ok = QInputDialog.getText(
            self,
            "Enter the name for the new action",
            "Action name:",
            QLineEdit.Normal,
            name,
        )
        if ok and name:
            if not isfile(context):
                QMessageBox.critical(
                    self, "Error", "Target action must be a local file.", QMessageBox.Ok
                )
                return 1, "Target action must be a local file"

            icon_filepath, _filter = QFileDialog.getOpenFileName(
                self, "Choose Icon", config.ASSETS_DIR
            )
            if not icon_filepath:
                icon_filepath = join(config.ASSETS_DIR, "downloads.png")
            settings.pushItem(
                {
                    "type": "file",
                    "name": name,
                    "desc": name,
                    "path": context,
                    "icon": icon_filepath,
                }
            )
            utils.propagateEvent(self, QEvent(events.RELOAD_WIDGETS))
            return 0, context

    def onCreateFolderAction(self):
        """
        Create a new DirTarget.
        """
        context = QFileDialog.getExistingDirectory(
            self, "Choose a directory", expanduser("~")
        )
        context = str(context)
        if not context:
            return 1, "Missing argument"
        name = os.path.basename(context)
        name, ok = QInputDialog.getText(
            self,
            "Enter the name for the new action",
            "Action name:",
            QLineEdit.Normal,
            name,
        )
        if ok and name:
            if not isdir(context):
                QMessageBox.critical(
                    self, "Error", "Target should be a local directory.", QMessageBox.Ok
                )
                return 1, "Target action must be a local directory"

            icon_filepath, _filter = QFileDialog.getOpenFileName(
                self, "Choose Icon", config.ASSETS_DIR
            )
            if not icon_filepath:
                icon_filepath = join(config.ASSETS_DIR, "downloads.png")
            settings.pushItem(
                {
                    "type": "dir",
                    "name": name,
                    "desc": name,
                    "path": context,
                    "icon": icon_filepath,
                }
            )
            utils.propagateEvent(self, QEvent(events.RELOAD_WIDGETS))
            return 0, context

    def onCreateFromTemplate(self):
        """
        TODO: make templates for standard actions like emails, twitter, etc.
        """
        pass

    def mousePressEvent(self, event):
        self.contextMenuEvent(event)

    def mouseDoubleClickEvent(self, event):
        pass

    def handle(self, context):
        pass


class DropTitleBar(QDialog):
    """
    TitleBar widget for our custom dialog.
    """

    def __init__(self, parent, title):
        QWidget.__init__(self, parent)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAutoFillBackground(True)
        self.setBackgroundRole(QPalette.Highlight)

        minmax = QToolButton(self)
        minmax.setIcon(QIcon(join(config.ASSETS_DIR, "minimize_window_white.png")))
        minmax.setMinimumHeight(10)
        minmax.setWindowOpacity(0.5)
        minmax.clicked.connect(self.minimax)
        minmax.setAttribute(Qt.WA_MacShowFocusRect, 0)

        close = QToolButton(self)
        close.setIcon(QIcon(join(config.ASSETS_DIR, "close_window_white.png")))
        close.setMinimumHeight(10)
        close.setWindowOpacity(0.5)
        close.clicked.connect(self.close)
        close.setAttribute(Qt.WA_MacShowFocusRect, 0)

        self.label = QLabel(self)
        self.label.setText(title)

        hbox = QHBoxLayout(self)
        hbox.addWidget(self.label)
        hbox.addWidget(minmax)
        hbox.addWidget(close)
        hbox.insertStretch(1, 500)
        hbox.setSpacing(0)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)

    def contextMenuEvent(self, event):
        self.parent().contextMenuEvent(event)

    def setTitle(self, title):
        self.label.setText(title)

    def minimax(self):
        utils.propagateEvent(self, QEvent(events.COLLAPSE_WINDOW))

    def close(self):
        utils.propagateEvent(self, QEvent(events.CLOSE_WINDOW))

    def reject(self):
        return


class DropTargetGrid(QWidget):
    """
    The body of our custom dialog which contains a grid of DropTargets.
    """

    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self.grid_layout = QGridLayout(self)
        self.settings = QSettings()
        self.reload()

    def event(self, evt):
        if evt.type() == events.RELOAD_WIDGETS:
            self.reload()
        return super(DropTargetGrid, self).event(evt)

    def reload(self):
        logging.info("Reloading grid items")
        layout = self.grid_layout
        utils.clearLayout(layout)
        items = settings.readItems()
        items.insert(
            0,
            {
                "type": "builtin",
                "name": "Create Action",
                "path": "",
                "icon": join(config.ASSETS_DIR, "plus_white.png"),
                "desc": "Create a new drop action",
            },
        )
        total_items = len(items)
        # Try to make the grid as close to a square as possible, but favour horizontal rectangles
        root = math.sqrt(total_items)
        rows = int(root)
        if rows == 0:
            rows = 1
        cols = int(total_items / rows) + 1

        item_idx = 0
        for j in range(rows):
            for i in range(cols):
                if item_idx >= total_items:
                    break
                conf = items[item_idx].copy()
                conf["index"] = item_idx - 1
                layout.addWidget(self.instantiateWidget(conf), j, i)
                item_idx = item_idx + 1

    def instantiateWidget(self, conf):
        """
        A helper method to instantiat DropWidgets depeneding on the type.
        """
        m = sys.modules[__name__]  # current module
        widget_classes = {
            "dir": getattr(m, "DirTarget"),
            "file": getattr(m, "FileTarget"),
            "builtin": getattr(m, "CreateTarget"),
        }
        widget_class = widget_classes[conf["type"]]
        widget = widget_class(self, conf)
        return widget


class DropFrame(QFrame):
    def __init__(self, parent=None):
        QFrame.__init__(self, parent)
        self.setFrameShape(QFrame.StyledPanel)

        self.titlebar = DropTitleBar(self, "Droptopus")
        self.content = DropTargetGrid(self)

        vbox = QVBoxLayout(self)
        vbox.addWidget(self.titlebar)
        vbox.addWidget(self.content)
        vbox.setContentsMargins(0, 0, 0, 0)
        vbox.setSpacing(0)
        self.setLayout(vbox)

    def sizeHint(self):
        tbs = self.titlebar.sizeHint()
        return QSize(tbs.width(), tbs.height()).__add__(self.content.sizeHint())

    def reload(self):
        self.content.reload()


def showError(text, details):
    msg = QMessageBox()
    msg.setIcon(QMessageBox.Critical)

    msg.setWindowTitle("Error")
    msg.setText(text)
    """
   msg.setInformativeText("This is additional information")
   """
    msg.setDetailedText("The details are as follows: " + details)
    msg.setStandardButtons(QMessageBox.Ok)
    retval = msg.exec_()
    print("value of pressed message box button:", retval)


def subprocessCall(args):
    cmd = args[0:1]
    filepath = args[1:2]
    ret = 0

    try:
        ret = subprocess.check_call(args)
    except subprocess.CalledProcessError:
        showError(
            "Error executing subprocess call",
            "The subprocess failed for the following command " + "".join(args),
        )
    except OSError:
        showError("Could not find executable", cmd)

    return ret, filepath



from setuptools import setup, find_packages
from os import path
exec(open('droptopus/version.py').read())

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# This call to setup() does all the work
setup(
    name="droptopus",
    version=__version__,
    description="Droptopus is a drag'n'drop router, which routes dropped objects to the designated actions",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ArtBIT/Droptopus",
    author="Djordje Ungar",
    author_email="mail@djordjeungar.com",
    license="MIT",
    classifiers=[
        # https://pypi.org/classifiers/
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    keywords="sample setuptools development",
    packages=["droptopus"],
    package_data={
        "droptopus": ["assets/*.png"]
    },
    include_package_data=True,
    install_requires=[
        "PyQt5",
        "python-magic",
        "requests>=2.21.0",
        "urllib3==1.24.1",
    ],
    project_urls={
        'Bug Reports': 'https://github.com/ArtBIT/Droptopus/issues',
        'Say Thanks!': 'http://saythanks.io/to/ArtBIT',
        'Source': 'https://github.com/ArtBIT/Droptopus/',
    },
    entry_points={
        'console_scripts': [
            'droptopus=droptopus:main',
        ],
    },
)
